@@ -12,11 +12,12 @@ import shutil
 # local imports
 from dotdrop.logger import Logger
 from dotdrop.linktypes import LinkTypes
-import dotdrop.utils as utils
+from dotdrop import utils
 from dotdrop.exceptions import UndefinedException
 
 
 class Installer:
+    """dotfile installer"""
 
     def __init__(self, base='.', create=True, backup=True,
                  dry=False, safe=False, workdir='~/.config/dotdrop',
@@ -57,7 +58,7 @@ class Installer:
         # when using install_to_tmp for comparing
         self.comparing = False
 
-        self.log = Logger()
+        self.log = Logger(debug=self.debug)
 
     ########################################################
     # public methods
@@ -65,7 +66,7 @@ class Installer:
 
     def install(self, templater, src, dst, linktype,
                 actionexec=None, noempty=False,
-                ignore=[], is_template=True,
+                ignore=None, is_template=True,
                 chmod=None, force_chmod=False):
         """
         install src to dst
@@ -88,14 +89,12 @@ class Installer:
         """
         if not src or not dst:
             # fake dotfile
-            if self.debug:
-                self.log.dbg('fake dotfile installed')
+            self.log.dbg('fake dotfile installed')
             self._exec_pre_actions(actionexec)
             return True, None
-        if self.debug:
-            msg = 'installing \"{}\" to \"{}\" (link: {})'
-            self.log.dbg(msg.format(src, dst, str(linktype)))
-        src, dst, cont, err = self._check_paths(src, dst, chmod)
+        msg = 'installing \"{}\" to \"{}\" (link: {})'
+        self.log.dbg(msg.format(src, dst, str(linktype)))
+        src, dst, cont, err = self._check_paths(src, dst)
         if not cont:
             return self._log_install(cont, err)
 
@@ -110,54 +109,53 @@ class Installer:
         # install to temporary dir
         # and ignore any actions
         if self.totemp:
-            r, err, _ = self.install_to_temp(templater, self.totemp,
-                                             src, dst, is_template=is_template,
-                                             chmod=chmod, ignore=ignore)
-            return self._log_install(r, err)
+            ret, err, _ = self.install_to_temp(templater,
+                                               self.totemp,
+                                               src, dst,
+                                               is_template=is_template,
+                                               chmod=chmod,
+                                               ignore=ignore)
+            return self._log_install(ret, err)
 
         isdir = os.path.isdir(src)
-        if self.debug:
-            self.log.dbg('install {} to {}'.format(src, dst))
-            self.log.dbg('\"{}\" is a directory: {}'.format(src, isdir))
+        self.log.dbg('install {} to {}'.format(src, dst))
+        self.log.dbg('\"{}\" is a directory: {}'.format(src, isdir))
 
         if linktype == LinkTypes.NOLINK:
             # normal file
             if isdir:
-                r, err = self._copy_dir(templater, src, dst,
-                                        actionexec=actionexec,
-                                        noempty=noempty, ignore=ignore,
-                                        is_template=is_template,
-                                        chmod=chmod)
+                ret, err = self._copy_dir(templater, src, dst,
+                                          actionexec=actionexec,
+                                          noempty=noempty, ignore=ignore,
+                                          is_template=is_template)
             else:
-                r, err = self._copy_file(templater, src, dst,
-                                         actionexec=actionexec,
-                                         noempty=noempty, ignore=ignore,
-                                         is_template=is_template,
-                                         chmod=chmod)
+                ret, err = self._copy_file(templater, src, dst,
+                                           actionexec=actionexec,
+                                           noempty=noempty, ignore=ignore,
+                                           is_template=is_template)
         elif linktype == LinkTypes.LINK:
             # symlink
-            r, err = self._link(templater, src, dst,
-                                actionexec=actionexec,
-                                is_template=is_template)
+            ret, err = self._link(templater, src, dst,
+                                  actionexec=actionexec,
+                                  is_template=is_template,
+                                  ignore=ignore)
         elif linktype == LinkTypes.LINK_CHILDREN:
             # symlink direct children
             if not isdir:
-                if self.debug:
-                    msg = 'symlink children of {} to {}'
-                    self.log.dbg(msg.format(src, dst))
+                msg = 'symlink children of {} to {}'
+                self.log.dbg(msg.format(src, dst))
                 err = 'source dotfile is not a directory: {}'.format(src)
-                r = False
+                ret = False
             else:
-                r, err = self._link_children(templater, src, dst,
-                                             actionexec=actionexec,
-                                             is_template=is_template,
-                                             ignore=ignore)
+                ret, err = self._link_children(templater, src, dst,
+                                               actionexec=actionexec,
+                                               is_template=is_template,
+                                               ignore=ignore)
 
-        if self.debug:
-            self.log.dbg('before chmod: {} err:{}'.format(r, err))
+        self.log.dbg('before chmod: {} err:{}'.format(ret, err))
 
         if self.dry:
-            return self._log_install(r, err)
+            return self._log_install(ret, err)
 
         # handle chmod
         # - on success (r, not err)
@@ -165,7 +163,7 @@ class Installer:
         # but not when
         # - error (not r, err)
         # - aborted (not r, err)
-        if (r or (not r and not err)):
+        if os.path.exists(dst) and (ret or (not ret and not err)):
             if not chmod:
                 chmod = utils.get_file_perm(src)
             dstperms = utils.get_file_perm(dst)
@@ -173,21 +171,21 @@ class Installer:
                 # apply mode
                 msg = 'chmod {} to {:o}'.format(dst, chmod)
                 if not force_chmod and self.safe and not self.log.ask(msg):
-                    r = False
+                    ret = False
                     err = 'aborted'
                 else:
                     if not self.comparing:
                         self.log.sub('chmod {} to {:o}'.format(dst, chmod))
                     if utils.chmod(dst, chmod, debug=self.debug):
-                        r = True
+                        ret = True
                     else:
-                        r = False
+                        ret = False
                         err = 'chmod failed'
 
-        return self._log_install(r, err)
+        return self._log_install(ret, err)
 
     def install_to_temp(self, templater, tmpdir, src, dst,
-                        is_template=True, chmod=None, ignore=[]):
+                        is_template=True, chmod=None, ignore=None):
         """
         install a dotfile to a tempdir
 
@@ -202,11 +200,11 @@ class Installer:
         return
         - success, error-if-any, dotfile-installed-path
         """
-        if self.debug:
-            self.log.dbg('tmp install {} (defined dst: {})'.format(src, dst))
-        src, dst, cont, err = self._check_paths(src, dst, chmod)
+        self.log.dbg('tmp install {} (defined dst: {})'.format(src, dst))
+        src, dst, cont, err = self._check_paths(src, dst)
         if not cont:
-            return self._log_install(cont, err)
+            self._log_install(cont, err)
+            return cont, err, None
 
         ret = False
         tmpdst = ''
@@ -228,9 +226,8 @@ class Installer:
                                 LinkTypes.NOLINK,
                                 is_template=is_template,
                                 chmod=chmod, ignore=ignore)
-        if self.debug:
-            if ret:
-                self.log.dbg('tmp installed in {}'.format(tmpdst))
+        if ret:
+            self.log.dbg('tmp installed in {}'.format(tmpdst))
 
         # restore flags
         self.dry = drysaved
@@ -246,7 +243,7 @@ class Installer:
     ########################################################
 
     def _link(self, templater, src, dst, actionexec=None,
-              is_template=True):
+              is_template=True, ignore=None):
         """
         install link:link
 
@@ -257,22 +254,22 @@ class Installer:
         - False, 'aborted'    : user aborted
         """
         if is_template:
-            if self.debug:
-                self.log.dbg('is a template')
-                self.log.dbg('install to {}'.format(self.workdir))
+            self.log.dbg('is a template')
+            self.log.dbg('install to {}'.format(self.workdir))
             tmp = self._pivot_path(dst, self.workdir, striphome=True)
-            r, err = self.install(templater, src, tmp,
-                                  LinkTypes.NOLINK,
-                                  actionexec=actionexec,
-                                  is_template=is_template)
-            if not r and not os.path.exists(tmp):
-                return r, err
+            ret, err = self.install(templater, src, tmp,
+                                    LinkTypes.NOLINK,
+                                    actionexec=actionexec,
+                                    is_template=is_template,
+                                    ignore=ignore)
+            if not ret and not os.path.exists(tmp):
+                return ret, err
             src = tmp
-        r, err = self._symlink(src, dst, actionexec=actionexec)
-        return r, err
+        ret, err = self._symlink(src, dst, actionexec=actionexec)
+        return ret, err
 
     def _link_children(self, templater, src, dst,
-                       actionexec=None, is_template=True, ignore=[]):
+                       actionexec=None, is_template=True, ignore=None):
         """
         install link:link_children
 
@@ -314,26 +311,24 @@ class Installer:
             subdst = dsts[i]
 
             if utils.must_ignore([subsrc, subdst], ignore, debug=self.debug):
-                if self.debug:
-                    self.log.dbg(
-                        'ignoring install of {} to {}'.format(src, dst),
-                    )
+                self.log.dbg(
+                    'ignoring install of {} to {}'.format(src, dst),
+                )
                 continue
 
-            if self.debug:
-                self.log.dbg('symlink child {} to {}'.format(subsrc, subdst))
+            self.log.dbg('symlink child {} to {}'.format(subsrc, subdst))
 
             if is_template:
-                if self.debug:
-                    self.log.dbg('child is a template')
-                    self.log.dbg('install to {} and symlink'
-                                 .format(self.workdir))
+                self.log.dbg('child is a template')
+                self.log.dbg('install to {} and symlink'
+                             .format(self.workdir))
                 tmp = self._pivot_path(subdst, self.workdir, striphome=True)
-                r, e = self.install(templater, subsrc, tmp,
-                                    LinkTypes.NOLINK,
-                                    actionexec=actionexec,
-                                    is_template=is_template)
-                if not r and e and not os.path.exists(tmp):
+                ret2, err2 = self.install(templater, subsrc, tmp,
+                                          LinkTypes.NOLINK,
+                                          actionexec=actionexec,
+                                          is_template=is_template,
+                                          ignore=ignore)
+                if not ret2 and err2 and not os.path.exists(tmp):
                     continue
                 subsrc = tmp
 
@@ -364,11 +359,12 @@ class Installer:
         - False, 'aborted'    : user aborted
         """
         overwrite = not self.safe
+
         if os.path.lexists(dst):
+            # symlink exists
             if os.path.realpath(dst) == os.path.realpath(src):
                 msg = 'ignoring "{}", link already exists'.format(dst)
-                if self.debug:
-                    self.log.dbg(msg)
+                self.log.dbg(msg)
                 return False, None
             if self.dry:
                 self.log.dry('would remove {} and link to {}'.format(dst, src))
@@ -378,22 +374,29 @@ class Installer:
             msg = 'Remove "{}" for link creation?'.format(dst)
             if self.safe and not self.log.ask(msg):
                 return False, 'aborted'
+
+            # remove symlink
             overwrite = True
             try:
                 utils.removepath(dst)
-            except OSError as e:
-                err = 'something went wrong with {}: {}'.format(src, e)
+            except OSError as exc:
+                err = 'something went wrong with {}: {}'.format(src, exc)
                 return False, err
+
         if self.dry:
             self.log.dry('would link {} to {}'.format(dst, src))
             return True, None
+
         base = os.path.dirname(dst)
         if not self._create_dirs(base):
             err = 'error creating directory for {}'.format(dst)
             return False, err
-        r, e = self._exec_pre_actions(actionexec)
-        if not r:
-            return False, e
+
+        # execute pre-actions
+        ret, err = self._exec_pre_actions(actionexec)
+        if not ret:
+            return False, err
+
         # re-check in case action created the file
         if os.path.lexists(dst):
             msg = 'Remove "{}" for link creation?'.format(dst)
@@ -401,9 +404,11 @@ class Installer:
                 return False, 'aborted'
             try:
                 utils.removepath(dst)
-            except OSError as e:
-                err = 'something went wrong with {}: {}'.format(src, e)
+            except OSError as exc:
+                err = 'something went wrong with {}: {}'.format(src, exc)
                 return False, err
+
+        # create symlink
         os.symlink(src, dst)
         if not self.comparing:
             self.log.sub('linked {} to {}'.format(dst, src))
@@ -411,8 +416,7 @@ class Installer:
 
     def _copy_file(self, templater, src, dst,
                    actionexec=None, noempty=False,
-                   ignore=[], is_template=True,
-                   chmod=None):
+                   ignore=None, is_template=True):
         """
         install src to dst when is a file
 
@@ -422,28 +426,23 @@ class Installer:
         - False, None       : ignored
         - False, 'aborted'    : user aborted
         """
-        if self.debug:
-            self.log.dbg('deploy file: {}'.format(src))
-            self.log.dbg('ignore empty: {}'.format(noempty))
-            self.log.dbg('ignore pattern: {}'.format(ignore))
-            self.log.dbg('is_template: {}'.format(is_template))
-            self.log.dbg('no empty: {}'.format(noempty))
+        self.log.dbg('deploy file: {}'.format(src))
+        self.log.dbg('ignore empty: {}'.format(noempty))
+        self.log.dbg('ignore pattern: {}'.format(ignore))
+        self.log.dbg('is_template: {}'.format(is_template))
+        self.log.dbg('no empty: {}'.format(noempty))
+
+        # ignore file
+        if utils.must_ignore([src, dst], ignore, debug=self.debug):
+            self.log.dbg('ignoring install of {} to {}'.format(src, dst))
+            return False, None
 
         # check no loop
         if utils.samefile(src, dst):
             err = 'dotfile points to itself: {}'.format(dst)
             return False, err
 
-        if utils.must_ignore([src, dst], ignore, debug=self.debug):
-            if self.debug:
-                self.log.dbg('ignoring install of {} to {}'.format(src, dst))
-            return False, None
-
-        if utils.samefile(src, dst):
-            # loop
-            err = 'dotfile points to itself: {}'.format(dst)
-            return False, err
-
+        # check source file exists
         if not os.path.exists(src):
             err = 'source dotfile does not exist: {}'.format(src)
             return False, err
@@ -455,14 +454,13 @@ class Installer:
             saved = templater.add_tmp_vars(self._get_tmp_file_vars(src, dst))
             try:
                 content = templater.generate(src)
-            except UndefinedException as e:
-                return False, str(e)
+            except UndefinedException as exc:
+                return False, str(exc)
             finally:
                 templater.restore_vars(saved)
             # test is empty
             if noempty and utils.content_empty(content):
-                if self.debug:
-                    self.log.dbg('ignoring empty template: {}'.format(src))
+                self.log.dbg('ignoring empty template: {}'.format(src))
                 return False, None
             if content is None:
                 err = 'empty template {}'.format(src)
@@ -471,8 +469,7 @@ class Installer:
         # write the file
         ret, err = self._write(src, dst,
                                content=content,
-                               actionexec=actionexec,
-                               chmod=chmod)
+                               actionexec=actionexec)
         if ret and not err:
             if not self.dry and not self.comparing:
                 self.log.sub('install {} to {}'.format(src, dst))
@@ -480,7 +477,7 @@ class Installer:
 
     def _copy_dir(self, templater, src, dst,
                   actionexec=None, noempty=False,
-                  ignore=[], is_template=True, chmod=None):
+                  ignore=None, is_template=True):
         """
         install src to dst when is a directory
 
@@ -490,57 +487,74 @@ class Installer:
         - False, None       : ignored
         - False, 'aborted'    : user aborted
         """
-        if self.debug:
-            self.log.dbg('deploy dir {}'.format(src))
+        self.log.dbg('deploy dir {}'.format(src))
         # default to nothing installed and no error
         ret = False, None
 
-        # create the directory anyway
-        if not self._create_dirs(dst):
-            err = 'creating directory for {}'.format(dst)
-            return False, err
-
         # handle all files in dir
         for entry in os.listdir(src):
-            f = os.path.join(src, entry)
-            if self.debug:
-                self.log.dbg('deploy sub from {}: {}'.format(dst, entry))
-            if not os.path.isdir(f):
+            fpath = os.path.join(src, entry)
+            self.log.dbg('deploy sub from {}: {}'.format(dst, entry))
+            if not os.path.isdir(fpath):
                 # is file
-                res, err = self._copy_file(templater, f,
+                res, err = self._copy_file(templater, fpath,
                                            os.path.join(dst, entry),
                                            actionexec=actionexec,
                                            noempty=noempty,
                                            ignore=ignore,
-                                           is_template=is_template,
-                                           chmod=None)
+                                           is_template=is_template)
                 if not res and err:
                     # error occured
-                    ret = res, err
-                    break
-                elif res:
+                    return res, err
+
+                if res:
                     # something got installed
                     ret = True, None
             else:
                 # is directory
-                res, err = self._copy_dir(templater, f,
+                res, err = self._copy_dir(templater, fpath,
                                           os.path.join(dst, entry),
                                           actionexec=actionexec,
                                           noempty=noempty,
                                           ignore=ignore,
-                                          is_template=is_template,
-                                          chmod=None)
+                                          is_template=is_template)
                 if not res and err:
                     # error occured
-                    ret = res, err
-                    break
-                elif res:
+                    return res, err
+
+                if res:
                     # something got installed
                     ret = True, None
         return ret
 
+    @classmethod
+    def _write_content_to_file(cls, content, src, dst):
+        """write content to file"""
+
+        if content:
+            # write content the file
+            try:
+                with open(dst, 'wb') as file:
+                    file.write(content)
+                shutil.copymode(src, dst)
+            except NotADirectoryError as exc:
+                err = 'opening dest file: {}'.format(exc)
+                return False, err
+            except OSError as exc:
+                return False, str(exc)
+            except TypeError as exc:
+                return False, str(exc)
+        else:
+            # copy file
+            try:
+                shutil.copyfile(src, dst)
+                shutil.copymode(src, dst)
+            except OSError as exc:
+                return False, str(exc)
+        return True, None
+
     def _write(self, src, dst, content=None,
-               actionexec=None, chmod=None):
+               actionexec=None):
         """
         copy dotfile / write content to file
 
@@ -556,22 +570,22 @@ class Installer:
             return True, None
 
         if os.path.lexists(dst):
+            # file/symlink exists
             try:
                 os.stat(dst)
-            except OSError as e:
-                if e.errno == errno.ENOENT:
+            except OSError as exc:
+                if exc.errno == errno.ENOENT:
                     # broken symlink
                     err = 'broken symlink {}'.format(dst)
                     return False, err
 
             if self.diff:
                 if not self._is_different(src, dst, content=content):
-                    if self.debug:
-                        self.log.dbg('{} is the same'.format(dst))
+                    self.log.dbg('{} is the same'.format(dst))
                     return False, None
+
             if self.safe:
-                if self.debug:
-                    self.log.dbg('change detected for {}'.format(dst))
+                self.log.dbg('change detected for {}'.format(dst))
                 if self.showdiff:
                     # get diff
                     self._show_diff_before_write(src, dst,
@@ -579,48 +593,38 @@ class Installer:
                 if not self.log.ask('Overwrite \"{}\"'.format(dst)):
                     return False, 'aborted'
                 overwrite = True
-        if self.backup and os.path.lexists(dst):
-            self._backup(dst)
+
+            if self.backup:
+                self._backup(dst)
+
+        # create hierarchy
         base = os.path.dirname(dst)
         if not self._create_dirs(base):
             err = 'creating directory for {}'.format(dst)
             return False, err
-        r, e = self._exec_pre_actions(actionexec)
-        if not r:
-            return False, e
-        if self.debug:
-            self.log.dbg('install file to \"{}\"'.format(dst))
-        # re-check in case action created the file
-        if self.safe and not overwrite and os.path.lexists(dst):
-            if not self.log.ask('Overwrite \"{}\"'.format(dst)):
-                self.log.warn('ignoring {}'.format(dst))
-                return False, 'aborted'
 
-        if content:
-            # write content the file
-            try:
-                with open(dst, 'wb') as f:
-                    f.write(content)
-                shutil.copymode(src, dst)
-            except NotADirectoryError as e:
-                err = 'opening dest file: {}'.format(e)
-                return False, err
-            except Exception as e:
-                return False, str(e)
-        else:
-            # copy file
-            try:
-                shutil.copyfile(src, dst)
-                shutil.copymode(src, dst)
-            except Exception as e:
-                return False, str(e)
-        return True, None
+        # execute pre actions
+        ret, err = self._exec_pre_actions(actionexec)
+        if not ret:
+            return False, err
+
+        self.log.dbg('install file to \"{}\"'.format(dst))
+        # re-check in case action created the file
+        if self.safe and not overwrite and \
+                os.path.lexists(dst) and \
+                not self.log.ask('Overwrite \"{}\"'.format(dst)):
+            self.log.warn('ignoring {}'.format(dst))
+            return False, 'aborted'
+
+        # writing to file
+        return self._write_content_to_file(content, src, dst)
 
     ########################################################
     # helpers
     ########################################################
 
-    def _get_tmp_file_vars(self, src, dst):
+    @classmethod
+    def _get_tmp_file_vars(cls, src, dst):
         tmp = {}
         tmp['_dotfile_sub_abs_src'] = src
         tmp['_dotfile_sub_abs_dst'] = dst
@@ -635,11 +639,10 @@ class Installer:
         if content:
             tmp = utils.write_to_tmpfile(content)
             src = tmp
-        r = utils.fastdiff(src, dst)
-        if r:
-            if self.debug:
-                self.log.dbg('content differ')
-        return r
+        ret = utils.fastdiff(src, dst)
+        if ret:
+            self.log.dbg('content differ')
+        return ret
 
     def _show_diff_before_write(self, src, dst, content=None):
         """
@@ -668,18 +671,14 @@ class Installer:
     def _create_dirs(self, directory):
         """mkdir -p <directory>"""
         if not self.create and not os.path.exists(directory):
-            if self.debug:
-                self.log.dbg('no mkdir as \"create\" set to false in config')
+            self.log.dbg('no mkdir as \"create\" set to false in config')
             return False
         if os.path.exists(directory):
             return True
         if self.dry:
             self.log.dry('would mkdir -p {}'.format(directory))
             return True
-        if self.debug:
-            self.log.dbg('mkdir -p {}'.format(directory))
-        if not self.comparing:
-            self.log.sub('create directory {}'.format(directory))
+        self.log.dbg('mkdir -p {}'.format(directory))
 
         os.makedirs(directory, exist_ok=True)
         return os.path.exists(directory)
@@ -694,15 +693,13 @@ class Installer:
 
     def _pivot_path(self, path, newdir, striphome=False):
         """change path to be under newdir"""
-        if self.debug:
-            self.log.dbg('pivot new dir: \"{}\"'.format(newdir))
-            self.log.dbg('strip home: {}'.format(striphome))
+        self.log.dbg('pivot new dir: \"{}\"'.format(newdir))
+        self.log.dbg('strip home: {}'.format(striphome))
         if striphome:
             path = utils.strip_home(path)
         sub = path.lstrip(os.sep)
         new = os.path.join(newdir, sub)
-        if self.debug:
-            self.log.dbg('pivot \"{}\" to \"{}\"'.format(path, new))
+        self.log.dbg('pivot \"{}\" to \"{}\"'.format(path, new))
         return new
 
     def _exec_pre_actions(self, actionexec):
@@ -716,7 +713,10 @@ class Installer:
         return ret, err
 
     def _log_install(self, boolean, err):
-        """log installation process"""
+        """
+        log installation process
+        returns success, error-if-any
+        """
         if not self.debug:
             return boolean, err
         if boolean:
@@ -728,7 +728,7 @@ class Installer:
                 self.log.dbg('install: IGNORED')
         return boolean, err
 
-    def _check_paths(self, src, dst, chmod):
+    def _check_paths(self, src, dst):
         """
         check and normalize param
         returns <src>, <dst>, <continue>, <error>
@@ -736,8 +736,7 @@ class Installer:
         # check both path are valid
         if not dst or not src:
             err = 'empty dst or src for {}'.format(src)
-            if self.debug:
-                self.log.dbg(err)
+            self.log.dbg(err)
             return None, None, False, err
 
         # normalize src and dst
